#include <config.h>
#include "hetero.h"
#include <stdio.h>
#include <math.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>

using namespace std;


////////////////////////////////////////////////////////////////////////////////
//
// launching point of the all the experiments;
//
////////////////////////////////////////////////////////////////////////////////

int main( int argc,char **argv) 
{
	// welcome;
	char welcome[] =
		"\n-------------------------------------------\n"
		"hnoc: Heterogeneous NoC Topology Synthesis\n"
		"Cristinel Ababei, Compiled "__DATE__"\n"
		"-------------------------------------------\n";
	printf("%s", welcome);



	// () runtime;
	// cpu time: method 1;
	clock_t start_clock, end_clock;
	clock_t diff_clock;
	start_clock = clock();
	assert(start_clock != (clock_t)(-1));
	// cpu time: method 2;
	struct tms t1, t2;
	times(&t1);
	// wall clock;
	timeval start_wall, end_wall; // sec and microsec;
	gettimeofday( &start_wall, 0);



	// (1)
	// create the main object inside which all experiments will be run;
	HETEROGENEOUS_NOC hetero;
	hetero.parse_command_arguments(argc, argv); // gets a "seed" too;
	hetero.setup_seed(); // setup the seed for C++ RNG;
	hetero.start_chronometer();
	hetero.print_setup_info(); // entertain user;



	// (2)
	// try 'n_fps' number of floorplans; I should have "best_fps" as a 
	// sketch array inside HETEROGENEOUS_NOC;
	vector <ONE_OF_THE_BEST_FLOOR_PLANS> best_fps;
	hetero.search_n_fps_floorplans( best_fps);



	// (3)
	// read the list of best FP's and for each: assign routers, find
	// routing paths using MCF, and refine buffer sizes; then finaly
	// simulate using vNOC;
	// the result of each simulation is stored inside each fp object
	// from the list;
	printf("\nThe %d best floorplans will be simulated.\n\n", best_fps.size());

	hetero.routers_assignment_routing_calculation_simulation( best_fps);



	// (4)
	// report results;
	// compute final results statistics and save in output files;
	hetero.calculate_final_results_statistics( argc, argv, best_fps);



	// () runtime;
	// cpu time;
	end_clock = clock();
	assert(end_clock != (clock_t)(-1));
	diff_clock = end_clock - start_clock;
	printf("hnoc total cputime (processor time, clock()) = %.3f sec\n",
		(double)diff_clock/CLOCKS_PER_SEC);
	// cpu time: method 2;
	times(&t2);
	printf("hnoc total cputime (processor time, tms) = %.3f sec\n",
		(double)(t2.tms_utime-t1.tms_utime)/HZ);
	// wall clock;
	gettimeofday( &end_wall, 0);
	double diff_sec_usec = end_wall.tv_sec - start_wall.tv_sec + 
		double(end_wall.tv_usec - start_wall.tv_usec) / 1000000.0;
	printf ("hnoc total walltime (wall clock time) = %.3f sec\n", diff_sec_usec);
	printf("\n");
}
